pub fn parse_u32(s: &str) -> Result<u32, ParseIntError>Expand description
Parses a u32 from a string that can be either hex (0x prefixed) or decimal.
pub fn parse_u32(s: &str) -> Result<u32, ParseIntError>Parses a u32 from a string that can be either hex (0x prefixed) or decimal.